;**********************************************************************

	list      p=12f629           ; list directive to define processor
	#include <p12f629.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT  ;_HS_OSC 


; Time Constants******************************************************
t1			EQU		.100		; time in ms = this value / 2 
t2			EQU		.15		; time in ms = this value / 10
t3			EQU		.100		; time in ms
;**********************************************************************


		cblock	0x22
s_GPIO	; shadowcopy of GPIO
dt1		; counter voor division factor  (phasetimer) start at 5 for t1, 1 for t2
ds1		; start value division factor.
dt2		; counter for division factor (comparator timer)
phase_status ;	1-4 for the various states. 
phase_counter
t3_counter
		endc


;**********************************************************************
w_temp        	EQU     0x20        ; variable used for context saving 
status_temp   	EQU     0x21        ; variable used for context saving
d1				EQU		.5			;division factor t1
d2				EQU		.1			;division factor t2
;**********************************************************************
		ORG     0x000             ; processor reset vector
		goto    main              ; go to beginning of program
;**********************************************************************
		ORG     0x004             ; interrupt vector location
		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
;-----------------------------------------------------------------------
		bcf		INTCON,T0IF			; reset timer interrupt flag...	
		movlw	.159
		addwf   TMR0,f
;-----------------------------------------------------------------------
		movf	s_GPIO,w
		movwf	GPIO

test_dt1:	;test whether action is going on for phase1+2, 
		decfsz	dt1,f
		goto test_dt2
		decfsz	phase_counter,f			 
		goto	phase_ready
		goto	test_phase_4

test_phase_4
		decfsz	phase_status,f		;phase follows resp. 4,3,2,1  
		goto	test_phase_1	
		movlw	.4
		movwf	phase_status
		movlw	t1
		movwf	phase_counter
		movlw	d1
		movwf	ds1
		bsf		s_GPIO,4
		goto	phase_ready
		
test_phase_1:
		decfsz	phase_status,f
		goto	test_phase_2
		movlw	.1
		movwf	phase_status
		movlw	t2
		movwf	phase_counter
		movlw	d2
		movwf	ds1
		bcf		s_GPIO,5
		goto	phase_ready

test_phase_2:
		decfsz	phase_status,f
		goto	phase_3	
		movlw	.2
		movwf	phase_status
		movlw	t1
		movwf	phase_counter
		movlw	d1
		movwf	ds1
		bsf		s_GPIO,5
		goto	phase_ready

phase_3:
		movlw	.3
		movwf	phase_status
		movlw	t2
		movwf	phase_counter
		movlw	d2
		movwf	ds1
		bcf		s_GPIO,4
		goto	phase_ready

phase_ready:
		movf	ds1,w
		movwf	dt1		

test_dt2:
		movf	t3_counter,f	;test zero
		btfsc	STATUS,Z
		goto	finished		;t3_counter==0
		decfsz	dt2,f
		goto	finished		;prescaler isn't 0
		decf	t3_counter,f
		movlw	.10
		movwf	dt2				;reset prescaler

;*****************************************************************************
finished:
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt

;********************************************************************************
main	;movlw	0x07		;turn comparator off
		movlw	b'00010100'			;turn comparator on (in op gp1, geen uit)
		movwf	CMCON
		bsf		STATUS,RP0
		movlw	b'00001010'	;0 = output
		movwf	TRISIO
		movlw	b'10001110'	;3,44V comparator
;		movlw	b'10101100'	;2,5V comparator
		movwf	VRCON
		bcf		STATUS,RP0

InitTimer:
        bsf     STATUS, RP0	; bank 1
		movlw   B'10001000'     ; 
		movwf   OPTION_REG      ;  
		call	3FFh
		movwf	OSCCAL
        bcf     STATUS, RP0	; bank 0
;***********************************************************************
		clrf	s_GPIO
		clrf	t3_counter
		movlw	.1
		movwf	dt1;
		movwf	dt2;
		movwf	ds1;
		movwf	phase_status ;	
		movwf	phase_counter;

;***********************************************************************		
		clrf    TMR0            ; reset timer (and prescaler!)
		movlw   B'10100000'     ; enable timer interrupt and GIE
		movwf   INTCON          ;  
;***********************************************************************		

Loop0:
Loop1:	movlw	.10
		movwf	dt2	
		movlw	t3
		movwf	t3_counter
Loop2:	btfss	CMCON,6
		goto	Loop1
		movf	t3_counter,f
		btfss	STATUS,Z
		goto	Loop2
		bsf		s_GPIO,0
Loop3:	movlw	.10
		movwf	dt2	
		movlw	t3
		movwf	t3_counter
Loop4:	btfsc	CMCON,6
		goto	Loop3
		movf	t3_counter,f
		btfss	STATUS,Z
		goto	Loop4
		bcf		s_GPIO,0
		goto 	Loop0

		END                       ; directive 'end of program'
